#import <Foundation/NSDictionary.h>
#import <Foundation/NSUserDefaults.h>
#import <AppKit/AppKit.h>

#define APP_NAME @"NoteBook"

int main(int argc, const char ** argv)
{
  return NSApplicationMain (argc, argv);
}
